@@ -847,5 +847,51 @@ export function openCompleteSSLCertificateInstallationModal() {
         }
     };
 }
+
+export function openDeployRemoteEndpointGroupModal() {
+    return {
+        type: OPEN_MODAL,
+        payload: {
+            component: 'deploy-remote-endpoint-group-modal',
+            options: {
+                title: 'Deploy Remote Endpoint Group',
+                size: 'small'
+            }
+        }
+    };
+}
+
+export function openEditEndpointGroupModal(groupName) {
+    return {
+        type: OPEN_MODAL,
+        payload: {
+            component: {
+                name: 'edit-endpoint-group-modal',
+                params: { groupName }
+            },
+            options: {
+                title: 'Edit Endpoint Group ',
+                size: 'small'
+            }
+        }
+    };
+}
+
+export function openOAuthAccessDeniedModal() {
+    return {
+        type: OPEN_MODAL,
+        payload: {
+            component: {
+                name: 'oauth-access-denied-modal',
+                params: { }
+            },
+            options: {
+                title: 'Access Denied',
+                size: 'xsmall',
+                severity: 'info'
+            }
+        }
+    };
+}
 /** INJECT:actionCreator **/
 
