@@ -20,7 +20,7 @@ module.exports = {
                 required: [
                     'name',
                     'is_managed',
-                    'host_count'
+                    'host_count',
                 ],
                 properties: {
                     name: {
@@ -40,7 +40,10 @@ module.exports = {
                                 $ref: 'common_api#/definitions/bigint'
                             }
                         }
-                    }
+                    },
+                    backingstore: {
+                        $ref: '#/definitions/backingstore_definition'
+                    },
                 }
             },
             reply: {
@@ -66,7 +69,10 @@ module.exports = {
                     },
                     target_bucket: {
                         type: 'string',
-                    }
+                    },
+                    backingstore: {
+                        $ref: '#/definitions/backingstore_definition'
+                    },
                 }
             },
             auth: {
@@ -229,7 +235,7 @@ module.exports = {
                     properties: {
                         service: {
                             type: 'string',
-                            enum: ['AWS', 'AZURE', 'S3_COMPATIBLE', 'GOOGLE', 'FLASHBLADE', 'NET_STORAGE']
+                            enum: ['AWS', 'AZURE', 'S3_COMPATIBLE', 'GOOGLE', 'FLASHBLADE', 'NET_STORAGE', 'IBM_COS']
                         },
                         read_count: {
                             type: 'integer'
@@ -323,7 +329,7 @@ module.exports = {
         scale_hosts_pool: {
             doc: 'Change the pool\'s underlaying host count',
             method: 'POST',
-             params: {
+            params: {
                 type: 'object',
                 required: ['name', 'host_count'],
                 properties: {
@@ -342,6 +348,18 @@ module.exports = {
     },
 
     definitions: {
+        backingstore_definition: {
+            type: 'object',
+            required: ['name', 'namespace'],
+            properties: {
+                name: {
+                    type: 'string',
+                },
+                namespace: {
+                    type: 'string',
+                }
+            }
+        },
         pool_definition: {
             type: 'object',
             required: ['name', 'nodes'],
@@ -605,6 +623,7 @@ module.exports = {
             enum: [
                 'HAS_NO_NODES',
                 'INITIALIZING',
+                'INITIALIZING_FAILED',
                 'DELETING',
                 'ALL_NODES_OFFLINE',
                 'SCALING',
