@@ -58,6 +58,7 @@ module.exports = {
             auth: {
                 account: false,
                 system: false,
+                anonymous: true,
             }
         },
 
@@ -295,6 +296,137 @@ module.exports = {
                 system: 'admin',
             }
         },
+
+        get_join_cluster_yaml: {
+            method: 'GET',
+            params: {
+                type: 'object',
+                properties: {
+                    region: { type: 'string' },
+                    endpoints: {
+                        type: 'object',
+                        required: [
+                            'min_count'
+                        ],
+                        properties: {
+                            min_count: {
+                                type: 'integer',
+                                minimum: 1
+                            },
+                            max_count: {
+                                type: 'integer',
+                                minimum: 1
+                            }
+                        }
+                    }
+                }
+            },
+            reply: {
+                type: 'string'
+            },
+            auth: {
+                system: 'admin',
+            }
+        },
+
+        update_endpoint_group: {
+            method: 'POST',
+            params: {
+                type: 'object',
+                required: [
+                    'group_name',
+                    'region',
+                    'endpoint_range'
+                ],
+                properties: {
+                    group_name: {
+                        type: 'string'
+                    },
+                    is_remote: {
+                        type: 'boolean'
+                    },
+                    region: {
+                        type: 'string'
+                    },
+                    endpoint_range: {
+                        type: 'object',
+                        required: ['min', 'max'],
+                        properties: {
+                            min: {
+                                type: 'integer',
+                                minimum: 1
+                            },
+                            max: {
+                                type: 'integer',
+                                minimum: 1
+                            }
+                        }
+                    }
+                }
+            },
+            auth: {
+                system: 'admin'
+            }
+        },
+
+        get_endpoints_history: {
+             method: 'GET',
+             params: {
+                 type: 'object',
+                 required: [
+                     'since'
+                 ],
+                 properties: {
+                     since: {
+                         idate: true
+                     },
+                     till: {
+                         idate: true
+                     },
+                     step: {
+                         type: 'integer'
+                     },
+                     groups: {
+                         type: 'array',
+                         items: {
+                             type: 'string'
+                         }
+                     }
+                 }
+             },
+             reply: {
+                 type: 'array',
+                 items: {
+                     type: 'object',
+                     properties: {
+                        timestamp: {
+                            idate: true
+                        },
+                        endpoint_count: {
+                            type: 'number'
+                        },
+                        cpu_count: {
+                            type: 'number'
+                        },
+                        cpu_usage: {
+                            type: 'number'
+                        },
+                        memory_usage: {
+                            type: 'number'
+                        },
+                        read_bytes: {
+                            type: 'number'
+                        },
+                        write_bytes: {
+                            type: 'number'
+                        }
+                     }
+                 }
+             },
+             auth: {
+                 system: 'admin'
+             }
+        }
     },
 
     definitions: {
@@ -323,7 +455,8 @@ module.exports = {
                 'objects',
                 'owner',
                 'node_version',
-                's3_endpoints'
+                's3_service',
+                'endpoint_groups'
             ],
             properties: {
                 name: {
@@ -494,23 +627,58 @@ module.exports = {
                         $ref: '#/definitions/restriction_enum'
                     }
                 },
-                s3_endpoints: {
-                    type: 'array',
-                    items: {
-                        type: 'object',
-                        required: ['kind', 'address'],
-                        properties: {
-                            kind: {
-                                type: 'string',
-                                enum: ['EXTERNAL', 'INTERNAL', 'LOOPBACK']
-                            },
-                            address: {
-                                type: 'string'
+                s3_service: {
+                    type: 'object',
+                    required: ['addresses'],
+                    properties: {
+                        addresses: {
+                            type: 'array',
+                            items: {
+                                type: 'object',
+                                required: ['kind', 'address'],
+                                properties: {
+                                    kind: {
+                                        type: 'string',
+                                        enum: ['EXTERNAL', 'INTERNAL', 'LOOPBACK']
+                                    },
+                                    address: {
+                                        type: 'string'
+                                    }
+                                }
                             }
                         }
                     }
+                },
+                endpoint_groups: {
+                    type: 'array',
+                    items: {
+                        type: 'object',
+                        required: [
+                            'group_name',
+                            'is_remote',
+                            'endpoint_count',
+                            'min_endpoint_count',
+                            'max_endpoint_count',
+                            'cpu_count',
+                            'cpu_usage',
+                            'memory_usage',
+                            'last_report_time'
+                        ],
+                        properties: {
+                            group_name: { type: 'string' },
+                            is_remote: { type: 'boolean' },
+                            region: { type: 'string' },
+                            endpoint_count: { type: 'number' },
+                            min_endpoint_count: { type: 'integer' },
+                            max_endpoint_count: { type: 'integer' },
+                            cpu_count: { type: 'number' },
+                            cpu_usage: { type: 'number' },
+                            memory_usage: { type: 'number' },
+                            last_report_time: { idate: true },
+                        }
+                    }
                 }
-            },
+            }
         },
 
         restriction_enum: {
