@@ -24,6 +24,7 @@ const data_chunk_schema = require('./schemas/data_chunk_schema');
 const data_chunk_indexes = require('./schemas/data_chunk_indexes');
 const data_block_schema = require('./schemas/data_block_schema');
 const data_block_indexes = require('./schemas/data_block_indexes');
+const config = require('../../../config');
 
 
 class MDStore {
@@ -132,6 +133,21 @@ class MDStore {
         mongo_utils.check_update_one(res, 'object');
     }
 
+    /**
+     * @param {nb.ID[]} object_ids
+     * @param {Object} [set_updates]
+     * @param {Object} [unset_updates]
+     */
+    async update_objects_by_ids(object_ids, set_updates, unset_updates) {
+        if (!object_ids || !object_ids.length) return;
+        dbg.log0('update_object_by_id:', object_ids.join(','), compact_updates(set_updates, unset_updates));
+        await this._objects.col().updateMany({
+            _id: {
+                $in: object_ids
+            }
+        }, compact_updates(set_updates, unset_updates));
+    }
+
     /**
      * @param {nb.ID} obj_id
      * @returns {Promise<nb.ObjectMD>}
@@ -403,7 +419,7 @@ class MDStore {
 
     /**
      * TODO define indexes used by find_objects()
-     * 
+     *
      * @typedef {Object} FindObjectsParams
      * @property {nb.ID} bucket_id
      * @property {string|RegExp} key
@@ -416,11 +432,11 @@ class MDStore {
      * @property {string} [sort]
      * @property {1|-1} [order]
      * @property {boolean} [pagination]
-     * 
+     *
      * @typedef {Object} FindObjectsReply
      * @property {nb.ObjectMD[]} objects
      * @property {{ non_paginated: Object, by_mode: Object }} counters
-     * 
+     *
      * @param {FindObjectsParams} params
      * @returns {Promise<FindObjectsReply>}
      */
@@ -496,6 +512,17 @@ class MDStore {
         };
     }
 
+    async find_unreclaimed_objects(limit) {
+        const results = await this._objects.col().find({
+            deleted: { $exists: true },
+            reclaimed: null
+        }, {
+            limit: Math.min(limit, 1000),
+            hint: 'deleted_unreclaimed_index',
+        }).toArray();
+        return results;
+    }
+
     async list_objects({
         bucket_id,
         delimiter,
@@ -731,6 +758,19 @@ class MDStore {
         return Boolean(obj);
     }
 
+    async all_buckets_with_completed_objects() {
+        return this._objects.col().distinct('bucket', {
+            // prefix for stored blob blocks information. TODO: move somwhere like config.js
+            key: { $not: /^\.noobaa_blob_blocks/ },
+            // partialFilterExpression:
+            deleted: null,
+            upload_started: null,
+        }, {
+            hint: 'version_seq_index',
+            sort: { bucket: 1 }
+        });
+    }
+
     async count_objects_of_bucket(bucket_id) {
         return this._objects.col().countDocuments({
             bucket: bucket_id,
@@ -1045,16 +1085,17 @@ class MDStore {
      * @param {nb.ObjectMD} obj
      * @returns {Promise<nb.ID[]>}
      */
-    async find_parts_chunk_ids(obj, none_deleted_only) {
+    async find_parts_chunk_ids(obj) {
         const find = {
             obj: obj._id,
+            deleted: null,
         };
-        if (none_deleted_only) find.deleted = null;
         return this._parts.col().find(find, {
                 projection: {
                     _id: 0,
-                    chunk: 1
-                }
+                    chunk: 1,
+                },
+                hint: 'obj_1_start_1'
             })
             .toArray()
             .then(parts => mongo_utils.uniq_ids(parts, 'chunk'));
@@ -1072,7 +1113,7 @@ class MDStore {
     }
 
     /**
-     * @param {nb.ID[]} chunk_ids 
+     * @param {nb.ID[]} chunk_ids
      * @returns {Promise<nb.ID[]>}
      */
     async find_parts_unreferenced_chunk_ids(chunk_ids) {
@@ -1110,7 +1151,7 @@ class MDStore {
     }
 
     /**
-     * @param {nb.ChunkSchemaDB[]} chunks 
+     * @param {nb.ChunkSchemaDB[]} chunks
      */
     async load_parts_objects_for_chunks(chunks) {
         if (!chunks || !chunks.length) return;
@@ -1356,7 +1397,7 @@ class MDStore {
     }
 
     /**
-     * @param {nb.ID[]} chunk_ids 
+     * @param {nb.ID[]} chunk_ids
      */
     async delete_chunks_by_ids(chunk_ids) {
         const delete_date = new Date();
@@ -1376,6 +1417,17 @@ class MDStore {
         });
     }
 
+    // Only for clean up in testing - Don't use unless you are sure!!!!
+    async delete_all_chunks_in_system() {
+        assert(config.test_mode, 'This function should be called only in test mode!');
+        const delete_date = new Date();
+        return this._chunks.col().updateMany({}, {
+            $set: {
+                deleted: delete_date,
+            },
+        });
+    }
+
     delete_object_by_id(object_id) {
         if (!object_id) return;
         return this._objects.col().updateOne({
@@ -1608,7 +1660,7 @@ class MDStore {
     }
 
     /**
-     * @param {nb.ID[]} block_ids 
+     * @param {nb.ID[]} block_ids
      */
     async delete_blocks_by_ids(block_ids) {
         const delete_date = new Date();
@@ -1776,7 +1828,7 @@ function sort_list_uploads_with_delimiter(a, b) {
 }
 
 /**
- * @param {string} id_str 
+ * @param {string} id_str
  * @returns {nb.ID}
  */
 function make_md_id(id_str) {
