@@ -18,7 +18,8 @@ const restrict = require('../../../platform_restrictions.json');
 const dbg = require('../../util/debug_module')(__filename);
 const cutil = require('../utils/clustering_utils');
 const config = require('../../../config');
-const BucketStatsStore = require('../analytic_services/bucket_stats_store').BucketStatsStore;
+const { BucketStatsStore } = require('../analytic_services/bucket_stats_store');
+const { EndpointStatsStore } = require('../analytic_services/endpoint_stats_store');
 const os_utils = require('../../util/os_utils');
 const { RpcError } = require('../../rpc');
 const nb_native = require('../../util/nb_native');
@@ -44,6 +45,9 @@ const chunk_config_utils = require('../utils/chunk_config_utils');
 const addr_utils = require('../../util/addr_utils');
 const url_utils = require('../../util/url_utils');
 const ssl_utils = require('../../util/ssl_utils');
+const yaml_utils = require('../../util/yaml_utils');
+const js_utils = require('../../util/js_utils');
+const { KubeStore } = require('../kube-store.js');
 
 const SYSLOG_INFO_LEVEL = 5;
 const SYSLOG_LOG_LOCAL1 = 'LOG_LOCAL1';
@@ -176,7 +180,8 @@ function new_system_defaults(name, owner_account_id) {
         upgrade_history: {
             successful_upgrades: [],
             last_failure: undefined
-        }
+        },
+        system_address: [],
     };
     return system;
 }
@@ -226,7 +231,7 @@ function new_system_changes(name, owner_account_id) {
             disabled: false
         }]
     );
-    const bucket = bucket_server.new_bucket_defaults(default_bucket_name, system._id, policy._id);
+    const bucket = bucket_server.new_bucket_defaults(default_bucket_name, system._id, policy._id, owner_account_id);
     return {
         insert: {
             systems: [system],
@@ -249,6 +254,9 @@ async function create_system(req) {
     if (system_store.data.systems.length > 20) {
         throw new Error('Too many created systems');
     }
+    if (system_store.data.systems.length > 0 && config.test_mode !== true) {
+        throw new Error('Cannot create multiple production systems');
+    }
 
     const {
         name,
@@ -315,14 +323,12 @@ async function create_system(req) {
 async function _get_cluster_info() {
     const cluster_info = await cluster_server.new_cluster_info({ address: "127.0.0.1" });
     if (cluster_info) {
-        const [dns_config] = await Promise.all([os_utils.get_dns_config()]);
-
+        const dns_config = await os_utils.get_dns_config();
         if (dns_config.dns_servers.length) {
             dbg.log0(`create_system: DNS servers were already configured in first install to`, dns_config.dns_servers);
             cluster_info.dns_servers = dns_config.dns_servers;
         }
     }
-
     return cluster_info;
 }
 
@@ -413,9 +419,9 @@ function read_system(req) {
             response => response.accounts
         ),
 
-        refresh_tiering_alloc: P.props(_.mapValues(system.buckets_by_name, bucket => node_allocator.refresh_tiering_alloc(bucket.tiering))),
+        refresh_system_alloc_unused: node_allocator.refresh_system_alloc(system),
 
-        deletable_buckets: P.props(_.mapValues(system.buckets_by_name, bucket => bucket_server.can_delete_bucket(system, bucket))),
+        undeletable_buckets: bucket_server.list_undeletable_buckets(),
 
         rs_status: system_store.get_local_cluster_info().is_clusterized ?
             MongoCtrl.get_hb_rs_status()
@@ -433,6 +439,7 @@ function read_system(req) {
             .then(res => res.functions),
 
         buckets_stats: BucketStatsStore.instance().get_all_buckets_stats({ system: system._id }),
+        endpoint_groups: _get_endpoint_groups()
 
     }).then(({
         nodes_aggregate_pool_no_cloud_and_mongo,
@@ -440,10 +447,11 @@ function read_system(req) {
         nodes_aggregate_pool_with_cloud_no_mongo,
         hosts_aggregate_pool,
         accounts,
-        deletable_buckets,
+        undeletable_buckets,
         rs_status,
         funcs,
-        buckets_stats
+        buckets_stats,
+        endpoint_groups
     }) => {
         const cluster_info = cutil.get_cluster_info(rs_status);
         const objects_sys = {
@@ -503,6 +511,7 @@ function read_system(req) {
         const base_address = addr_utils.get_base_address(system.system_address);
         const dns_name = net.isIP(base_address.hostname) === 0 ? base_address.hostname : undefined;
         const tiering_status_by_tier = {};
+        const undeletable_bucket_set = new Set(undeletable_buckets);
 
         return {
             name: system.name,
@@ -528,8 +537,8 @@ function read_system(req) {
                         bucket_stats: stats_by_bucket[bucket.name],
                     });
                     const bucket_name = bucket.name.unwrap();
-                    if (deletable_buckets[bucket_name]) {
-                        b.undeletable = deletable_buckets[bucket_name];
+                    if (undeletable_bucket_set.has(bucket_name)) {
+                        b.undeletable = 'NOT_EMPTY';
                     }
                     return b;
                 }),
@@ -577,7 +586,10 @@ function read_system(req) {
                 }
             },
             platform_restrictions: restrict[process.env.PLATFORM || 'dev'], // dev will be default for now
-            s3_endpoints: _list_s3_endpoints(system)
+            s3_service: {
+                addresses: _list_s3_addresses(system)
+            },
+            endpoint_groups
         };
     });
 }
@@ -877,7 +889,7 @@ async function _ensure_internal_structure(system_id) {
     if (!support_account) throw new Error('SUPPORT ACCOUNT DOES NOT EXIST');
     try {
         server_rpc.client.hosted_agents.create_pool_agent({
-            pool_name: `${config.INTERNAL_STORAGE_POOL_NAME}-${system}`
+            pool_name: `${config.INTERNAL_STORAGE_POOL_NAME}-${system_id}`
         }, {
             auth_token: auth_server.make_auth_token({
                 system_id,
@@ -890,6 +902,244 @@ async function _ensure_internal_structure(system_id) {
     }
 }
 
+async function get_join_cluster_yaml(req) {
+    const { region = '', endpoints = {} } = req.rpc_params;
+    const ep_min_count = endpoints.min_count || 1;
+    const ep_max_count = endpoints.max_count || ep_min_count;
+    if (ep_max_count < ep_min_count) {
+        throw new RpcError('BAD_REQUEST', 'endpoints.max_count cannot be lower then endpoints.min_count');
+    }
+
+    const operator_account = system_store.data.accounts.find(account =>
+        account.roles_by_system && // This will protect against support account
+        account.roles_by_system[req.system._id] &&
+        account.roles_by_system[req.system._id].includes('operator')
+    );
+    if (!operator_account) {
+        throw new RpcError('NO_OPERATOR_ACCOUNT', 'Cannot find operator account');
+    }
+
+    const joinSecret = {
+        apiVersion: 'v1',
+        kind: 'Secret',
+        metadata: {
+            name: 'join-secret',
+            namespace: config.REMOTE_NOOAA_NAMESPACE,
+            labels: {
+                app: 'noobaa'
+            }
+        },
+        type: 'Opaque',
+        stringData: {
+            auth_token: await auth_server.make_auth_token({
+                system_id: req.system._id,
+                account_id: operator_account._id,
+                role: 'operator'
+            }),
+            ...Object.fromEntries(
+                ['mgmt', 'bg', 'md', 'hosted_agents'].map(api_name => [
+                    `${api_name}_addr`,
+                    addr_utils.get_base_address(req.system.system_address, {
+                        api: api_name,
+                        hint: 'EXTERNAL',
+                        protocol: 'wss',
+                        secure: true
+                    }).toString()
+                ])
+            )
+        }
+    };
+
+    const noobaa = {
+        apiVersion: 'noobaa.io/v1alpha1',
+        kind: 'NooBaa',
+        metadata: {
+            name: 'noobaa',
+            namespace: config.REMOTE_NOOAA_NAMESPACE,
+            labels: {
+                app: 'noobaa'
+            }
+        },
+        spec: {
+            joinSecret: _.pick(joinSecret.metadata, ['name', 'namespace']),
+            region,
+            endpoints: {
+                minCount: ep_min_count,
+                maxCount: ep_max_count
+            }
+        }
+    };
+
+    return yaml_utils.stringify([
+        joinSecret,
+        noobaa
+    ]);
+}
+
+
+async function update_endpoint_group(req) {
+    const { group_name, is_remote, region, endpoint_range } = req.rpc_params;
+
+    const cluster = system_store.get_local_cluster_info();
+    const exists = (cluster.endpoint_groups || [])
+        .some(group => group.name === group_name);
+
+    if (exists) {
+        const group = cluster.endpoint_groups.find(grp => grp.name === group_name);
+        if (!_.isUndefined(is_remote)) {
+            if (group.is_remote !== is_remote) {
+                // We do not throw in order to not fail the noobaa operator.
+                dbg.warn('update_endpoint_group: Conflicted is_remote value of ',
+                    is_remote, ' for group: ', group, ' - aborting request');
+                return;
+            }
+
+        }
+
+        // call make_changes only if there are actual changes to make.
+        // this check fixes a bug where make_changes sends a load_system_store notification 
+        // to the operator, which in its own reconcile sends back update_endpoint_group, and so forth
+        if (group.region !== region || !_.isEqual(group.endpoint_range, endpoint_range)) {
+            await system_store.make_changes({
+                update: {
+                    clusters: [{
+                        $find: {
+                            _id: cluster._id,
+                            'endpoint_groups.name': group_name
+                        },
+                        $set: {
+                            'endpoint_groups.$.region': region,
+                            'endpoint_groups.$.endpoint_range': endpoint_range
+                        }
+                    }]
+                }
+            });
+        }
+
+    } else {
+        await system_store.make_changes({
+            update: {
+                clusters: [{
+                    _id: cluster._id,
+                    $push: {
+                        endpoint_groups: {
+                            name: group_name,
+                            is_remote,
+                            region,
+                            endpoint_range
+                        }
+                    }
+                }]
+            }
+        });
+    }
+
+    // Update the noobaa CRD if request was not originated from the operator
+    const account_roles = req.account.roles_by_system[req.system._id];
+    if (!account_roles.includes('operator') && !is_remote) {
+        try {
+            await KubeStore.instance.patch_noobaa({
+                spec: {
+                    region,
+                    endpoints: {
+                        minCount: endpoint_range.min,
+                        maxCount: endpoint_range.max
+                    }
+                }
+            });
+        } catch (err) {
+            dbg.error('update_endpoint_group: Could not update noobaa CRD, got', err);
+        }
+    }
+}
+
+const _get_group_accumulator = (group_map, group_name) =>
+    js_utils.map_get_or_create(group_map, group_name, () => ({
+        count: 0,
+        endpoint_count: 0,
+        cpu_count: 0,
+        cpu_usage: 0,
+        memory_usage: 0,
+        read_bytes: 0,
+        write_bytes: 0
+    }));
+
+async function get_endpoints_history(req) {
+    const monitor_interval = config.ENDPOINT_MONITOR_INTERVAL;
+    const { groups, step = 1 } = req.rpc_params;
+    const since = Math.ceil(req.rpc_params.since / monitor_interval) * monitor_interval;
+    const till = Math.floor(Math.min(req.rpc_params.till || Infinity, Date.now()) / monitor_interval) * monitor_interval;
+    const stepInMili = moment.duration(step || 1, 'hour').asMilliseconds(); // the rpc params unit is hours.
+    const [group_reports, bandwidth_reports] = await Promise.all([
+        EndpointStatsStore.instance.get_endpoint_group_reports({ since, till, groups }),
+        EndpointStatsStore.instance.get_bandwidth_reports({ since, till, endpoint_groups: groups })
+    ]);
+
+    // Sort by time.
+    group_reports.sort((a, b) => a.end_time - b.end_time);
+    bandwidth_reports.sort((a, b) => a.end_time - b.end_time);
+
+    let i = 0;
+    let j = 0;
+    const bins = [];
+    for (let start_time = since; start_time < till; start_time += stepInMili) {
+        const end_time = start_time + stepInMili;
+        const by_groups = new Map();
+        for (; i < group_reports.length; ++i) {
+            const report = group_reports[i];
+            if (report.end_time >= end_time) break;
+
+            const { endpoints, group_name } = report;
+            const acc = _get_group_accumulator(by_groups, group_name);
+            const memory_usage = _.sumBy(endpoints, ep_info =>
+                ep_info.memory.used / ep_info.memory.total
+            ) / endpoints.length;
+            acc.count += 1;
+            acc.endpoint_count += endpoints.length;
+            acc.cpu_count += _.sumBy(endpoints, ep_info => ep_info.cpu.count);
+            acc.cpu_usage += _.sumBy(endpoints, ep_info => ep_info.cpu.usage);
+            acc.memory_usage += memory_usage;
+        }
+
+        for (; j < bandwidth_reports.length; ++j) {
+            const report = bandwidth_reports[j];
+            if (report.end_time >= end_time) break;
+
+            const acc = _get_group_accumulator(by_groups, report.group_name);
+            acc.bandwidth_reports += 1;
+            acc.read_bytes += report.read_bytes;
+            acc.write_bytes += report.write_bytes;
+        }
+
+        const bin = {
+            timestamp: end_time,
+            endpoint_count: 0,
+            cpu_count: 0,
+            cpu_usage: 0,
+            memory_usage: 0,
+            read_bytes: 0,
+            write_bytes: 0
+        };
+
+        const desired_report_count = end_time <= till ?
+            (stepInMili / monitor_interval) :
+            ((till - start_time) / monitor_interval);
+
+        for (const acc of by_groups.values()) {
+            bin.endpoint_count += acc.endpoint_count / desired_report_count;
+            bin.cpu_count += acc.cpu_count / desired_report_count;
+            bin.cpu_usage += acc.cpu_usage / desired_report_count;
+            bin.memory_usage += (acc.memory_usage / desired_report_count) / by_groups.size;
+            bin.read_bytes += acc.read_bytes;
+            bin.write_bytes += acc.write_bytes;
+        }
+
+        bins.push(bin);
+    }
+
+    return bins;
+}
+
 // UTILS //////////////////////////////////////////////////////////
 
 function get_system_info(system, get_id) {
@@ -908,7 +1158,7 @@ function find_account_by_email(req) {
     return account;
 }
 
-function _list_s3_endpoints(system) {
+function _list_s3_addresses(system) {
     if (process.platform === 'darwin') {
         return [{
             kind: 'LOOPBACK',
@@ -946,6 +1196,40 @@ function _list_s3_endpoints(system) {
         });
 }
 
+async function _get_endpoint_groups() {
+    const { endpoint_groups = [] } = system_store.get_local_cluster_info();
+    const till = Math.floor(Date.now() / config.ENDPOINT_MONITOR_INTERVAL) * config.ENDPOINT_MONITOR_INTERVAL;
+    const reports = await EndpointStatsStore.instance.get_endpoint_group_reports({
+        // Get information for group that are still registered (ignoring legacy groups)
+        groups: endpoint_groups.map(group => group.name),
+        // We go one full cycle behind a frame with data from all groups/endpoints.
+        since: till - config.ENDPOINT_MONITOR_INTERVAL,
+        till: till
+    });
+
+    const reports_by_group = _.keyBy(reports, report => report.group_name);
+    return endpoint_groups.map(group => {
+        const { end_time = -1, endpoints = [] } = reports_by_group[group.name] || {};
+        const ep_count = endpoints.length;
+        const memory_usage = ep_count > 0 ?
+            (_.sumBy(endpoints, ep_info => ep_info.memory.used / ep_info.memory.total) / ep_count) :
+            0;
+
+        return {
+            group_name: group.name,
+            is_remote: group.is_remote,
+            region: group.region,
+            endpoint_count: ep_count,
+            min_endpoint_count: group.endpoint_range.min,
+            max_endpoint_count: group.endpoint_range.max,
+            cpu_count: _.sumBy(endpoints, ep_info => ep_info.cpu.count),
+            cpu_usage: _.sumBy(endpoints, ep_info => ep_info.cpu.usage), // Can add 1.0 per cpu.
+            memory_usage,
+            last_report_time: end_time
+        };
+    });
+}
+
 // EXPORTS
 exports._init = _init;
 exports.is_initialized = is_initialized;
@@ -971,4 +1255,6 @@ exports.update_n2n_config = update_n2n_config;
 exports.attempt_server_resolve = attempt_server_resolve;
 exports.set_maintenance_mode = set_maintenance_mode;
 exports.set_webserver_master_state = set_webserver_master_state;
-
+exports.get_join_cluster_yaml = get_join_cluster_yaml;
+exports.update_endpoint_group = update_endpoint_group;
+exports.get_endpoints_history = get_endpoints_history;
